/************************************************************
 * Board Display Routines
 * Kenrick Mock 12/20/01
 *   added CountRunners, TimeoutScore 3/27/02
 ************************************************************/

#include <stdio.h>
#include <string.h>
#include "board.h"
#include "globals.h"


/************************************************************
 * CountRunners
 * Computes how many runners for the particular color
 ************************************************************/
int CountRunners(char board[DIM][DIM], char color)
{
 int i,j;
 int total=0;
 for (i=1; i<DIM; i+=2)
  for (j=1; j<DIM; j+=2) {
	if ((board[i][j]==B_RUNNER) && (color=='b')) total++;
	else if ((board[i][j]==W_RUNNER) && (color=='w')) total++;
  }
 return total;
}



/************************************************************
 * TimeoutScore
 * Computes score in advancement of runners
 ************************************************************/
int TimeoutScore(char board[DIM][DIM], char color)
{
 int i,j;
 int total=0;
 for (i=1; i<DIM; i+=2)
  for (j=1; j<DIM; j+=2) {
	if ((board[i][j]==B_RUNNER) && (color=='b')) total+=((j-1)/2);
	else if ((board[i][j]==W_RUNNER) && (color=='w')) total+=((13-j)/2);
  }
 return total;
}



/************************************************************
 * ShowBoard
 * Displays the board in beautiful ASCII.
 * Requires a display that can show approximately 30 vertical
 * lines, so a 25 line screen is too small to show the whole
 * board without scrolling.
 ************************************************************/
void ShowBoard(char board[DIM][DIM])
{
 int i,j;
 char c;

 for (j=1; j<DIM; j++) {
	printf("    /\\  /\\  /\\  /\\  /\\  /\\  /\\\n");
   	printf("%2d ",j);
	for (i=1; i<DIM; i+=2) {
		c=' ';
		if (board[i][j]!=EMPTY) c=board[i][j];
	   	printf("/ %c\\",c);
	}
	printf("\n");
	j++;
	printf("   \\  /\\  /\\  /\\  /\\  /\\  /\\  /\n");
	if (j<DIM) {
   		printf("%2d ",j);
		for (i=2; i<DIM; i+=2) {
			c=' ';
			if (board[i][j]!=EMPTY) c=board[i][j];
			printf(" \\/%c",c);
		}
		printf(" \\/\n");
	}
	else {
		printf("    \\/  \\/  \\/  \\/  \\/  \\/  \\/\n");
		printf("      2   4   6   8   10  12\n");
		printf("     1  3   5   7   9   11  13\n");
	}
 }
}

/************************************************************
 * InitBoard
 * Initialize the board array.
 * number 0 = empty
 * w = white runner,  @ = white guard
 * b = black runner,  B = black guard
 ************************************************************/
void InitBoard(char board[DIM][DIM])
{
 int i,j;
 for (i=1; i<DIM; i++) 
	for (j=1; j<DIM; j++) 
		board[i][j]=EMPTY;
 for (i=3; i<13; i+=2) {
	board[i][1]=B_RUNNER;
	board[i][13]=W_RUNNER;
 }
 for (i=4; i<12; i+=2) {
	board[i][2]=B_GUARD;
	board[i][12]=W_GUARD;
 }
}

