
/***************************************************************
 ************************** STOP !! ****************************
 ***************************************************************/

/*
   If you are looking at this file, it should only be for
   educational purposes.  If you want a code base to start
   from to use in your own program, use the CLIENT source
   code, not this code.  This code implements a server that
   is essentially just a pipe between two game programs.
 */

/* Name : gamemanager.c
 *
 * Revision 3/27/02 for nice -20, showing scores
 * Revised 3/3/02 for HiQUE by kjm
 * Revised 4/4/02 for time by kjm
 *
 * Notes:  Based upon quixo.cc manager by idr
 * Revision 1.2  1996/10/08 19:47:12  idr
 * Revision 1.1  1996/10/07 22:24:59  idr
 * Initial revision
 *
 */

#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <time.h>
#include "globals.h"
#include "move.h"
#include "board.h"

#define COMPUTER 0
#define HUMAN 1

void ShowUsage( const char * name );
void StartProgram( const char * program, FILE * fd[], const char * flags);

/*****************************************************************************/
/* Name : main()
 *
 * Notes:
 *
 */

int main( int argc, char ** argv )
{
    int player1, player2;
    FILE *fd_player1[2], *fd_player2[2];
    char board[DIM][DIM];          /* Main Game Board */
    int x1,x2,y1,y2;               /* Holds move coordinates */
    int turn=0;
    int iWinner;
    int bRunners, wRunners, bScore, wScore;
    double d;
    time_t t1,t2;

    if ( argc < 3 )
    {
        ShowUsage( argv[ 0 ] );
        exit( 1 );
    }

    /* Create the structures for talking to our two players. */
    if (!strcmp("-",argv[1])) {
	player1 = HUMAN;
    }
    else {
        player1 = COMPUTER;
    	StartProgram(argv[1], fd_player1, "black");
    }
    if (!strcmp("-",argv[2])) {
	player2 = HUMAN;
    }
    else {
        player2 = COMPUTER;
    	StartProgram(argv[2], fd_player2, "white");
    }

    /* Initialize our board and the UI */
    InitBoard(board);
    g_num_white = 5;
    g_num_black = 5;

    /* Play the game until somebody wins, or until they hit ^C. :) */
    while (!GameOver(board,0))
    {
	turn++;
 	bRunners = CountRunners(board,'b');
	wRunners = CountRunners(board,'w');
	bScore = TimeoutScore(board,'b') + bRunners;
	wScore = TimeoutScore(board,'w') + wRunners;
	fprintf(stdout,"Turn=%d. White Runners: %d, score: %d     Black Runners:%d, score: %d\n",turn,wRunners,wScore,bRunners,bScore);
	ShowBoard(board);

        /* Get the first player's move and do it.  If the move is not valid,
         * then display the move to the "console" and break out with player 2
         * as the winner.
         */
	t1 = time(NULL);
	if (player1==HUMAN) {
        	fprintf(stdout,"Player 1 (black), enter your move.\n");
		fscanf(stdin,"%d %d %d %d",&x1,&y1,&x2,&y2);
 	}
	else {
        	fprintf(stdout,"Waiting for computer player 1's (black) move.\n");
		fscanf(fd_player1[0],"%d %d %d %d",&x1,&y1,&x2,&y2);
	}
	t2 = time(NULL);
 	d = difftime(t2,t1);
	fprintf(stdout,"Player 1 moved : %d %d %d %d.  Time to move: %f seconds.\n", x1,y1,x2,y2,d);
	if (d > 20) fprintf(stdout,"*** WARNING : Move took over 20 seconds *** \n");
	/* Check to see if move is valid */
	if (!IsValid(board,'b',x1,y1,x2,y2)) {
		fprintf(stdout,"Player 1 entered an invalid move!");
	 	fprintf(stdout," Move= %d %d %d %d\n",x1,y1,x2,y2);
		fprintf(stdout," White wins!\n");
		goto EndOfGame;
	}
	else {
		ApplyMove(board,x1,y1,x2,y2,1);
	}
	turn++;
 	bRunners = CountRunners(board,'b');
	wRunners = CountRunners(board,'w');
	bScore = TimeoutScore(board,'b') + bRunners;
	wScore = TimeoutScore(board,'w') + wRunners;
	fprintf(stdout,"Turn=%d. White Runners: %d, score: %d     Black Runners:%d, score: %d\n",turn,wRunners,wScore,bRunners,bScore);

	ShowBoard(board);
	/* Check if this was a game winner */
	iWinner=GameOver(board,1);
	if (iWinner!=0) goto EndOfGame;

        /* Tell player 2 what player 1 did. */
	t1 = time(NULL);
	if (player2==COMPUTER) {
		fprintf(fd_player2[1],"%d %d %d %d\n", x1,y1,x2,y2);
       		fflush(fd_player2[1]);
		/* Get player 2's move */
        	fprintf(stdout,"Waiting for computer player 2's (white) move.\n");
		fscanf(fd_player2[0],"%d %d %d %d",&x1, &y1, &x2, &y2);
 	}
	else {
        	fprintf(stdout,"Player 2 (white), enter your move.\n");
		fscanf(stdin,"%d %d %d %d",&x1,&y1,&x2,&y2);
	}
	t2 = time(NULL);
	d = difftime(t2,t1);
	fprintf(stdout,"Player 2 moved : %d %d %d %d.  Time to move: %f seconds.\n", x1,y1,x2,y2,d);
	if (d > 20) fprintf(stdout,"*** WARNING : Move took over 20 seconds *** \n");
	if (!IsValid(board,'w',x1,y1,x2,y2)) {
		fprintf(stdout,"Player 2 entered an invalid move!");
	 	fprintf(stdout," Move= %d %d %d %d\n",x1,y1,x2,y2);
		fprintf(stdout," Black wins!\n");
		goto EndOfGame;
	}
	else {
		ApplyMove(board,x1,y1,x2,y2,1);
	}

        /* Tell player 1 what player 2 did. */
	if (player1==COMPUTER) {
		fprintf(fd_player1[1],"%d %d %d %d\n", x1,y1,x2,y2);
       		fflush(fd_player1[1]);
	}
    }

EndOfGame:

    /* Tell them who won, close up shop, and terminate. */
    ShowBoard(board);
    printf("\nGAME OVER.\n");
    GameOver(board,1);
    return( 0 );
}


/*****************************************************************************/
/* Name : StartProgram()
 *
 * Notes: This function starts a new process and runs the named program.  It
 *        fills fd[0] with a file handle that is tied to the new program's
 *        STDOUT (i.e., you can read the output of the child program through
 *        this file), and fills fd[1] with a file handle that is tied to the
 *        new program's STDIN.
 */

void StartProgram( const char * program, FILE* fd[], const char * player )
{
    int     pipeIn[ 2 ];        /* Provides STDIN for the CHILD.    */
    int     pipeOut[ 2 ];       /* Provides STDOUT for the CHILD.   */

    /* First, create the two pipes that we will use to communicate with the
     * child processes.
     */

    if ( pipe( pipeIn ) == -1 )
    {
        perror( "pipe( pipeIn )" );
        exit( 1 );
    }

    if ( pipe( pipeOut ) == -1 )
    {
        perror( "pipe( pipeOut )" );
        exit( 1 );
    }

    /* Create the child process! */
    if ( fork() == 0 )
    {
        /* Close the default STDIN and STDOUT file handles.  We want our
         * STDIN and STDOUT tied directly to our parent process.
         */

        close( STDIN_FILENO );
        close( STDOUT_FILENO );


        /* Close the write part of our STDIN pipe and the read part of our
         * STDOUT pipe.  As the child, we can't use these.
         */

        close( pipeIn[ 1 ] );
        close( pipeOut[ 0 ] );



        if ( dup2( pipeIn[ 0 ], STDIN_FILENO ) == -1 )
        {
            perror( "dup2( pipeIn[ 0 ], STDIN_FILENO )" );
            exit( 1 );
        }

        if ( dup2( pipeOut[ 1 ], STDOUT_FILENO ) == -1 )
        {
            perror( "dup2( pipeOut[ 1 ], STDOUT_FILENO )" );
            exit( 1 );
        }


        /* Run the specified program. */

        /* execl(program, program, player, NULL ); */
        execl("/usr/bin/nice", "-20", program, player, NULL ); 


        /* If we get to this point, it means that execl() failed.  Let's
         * print an error message and exit.
         */

        perror( "execl()" );
        exit( 1 );
    }

    /* Close the read part of the child's STDIN pipe and the out part of
     * the child's STDOUT pipe.  As the parent, we (can't) won't be using
     * these parts.
     */

    close( pipeIn[ 0 ] );
    close( pipeOut[ 1 ] );

    /* Convert the parts of the pipes that we will be using to normal
     * stdio style FILEs.
     */

    fd[ 0 ] = fdopen( pipeOut[ 0 ], "r" );
    if ( fd[ 0 ] == NULL )
    {
        perror( "fdopen" );
        exit( 1 );
    }

    fd[ 1 ] = fdopen( pipeIn[ 1 ], "w" );
    if ( fd[ 1 ] == NULL )
    {
        perror( "fdopen" );
        exit( 1 );
    }
}




/*****************************************************************************/
/* Name : ShowUsage()
 *
 * Notes: This function attempts to show the user how to properly run our
 *        little program.
 */

void ShowUsage( const char * name )
{
    fprintf( stderr, "\nUsage: %s player1_program player2_program\n" \
                     "If the player_program is \"-\", then the player " \
                     "actually a human player.\n",
             name );
    fprintf(stderr,"\nThe first player is black, the second is white.\n");
}

